import math
import random
import pygame

def Mod(num):
    if num < 0:num = -num
    return num


class Math_:
    def COS(self,ugl):
        return math.cos(ugl)

    def SIN(self,ugl):
        return math.sin(ugl) 

    def RAST(self,pos1=[],pos2=[]):
        if pos1[0]>pos2[0]:w = pos1[0]-pos2[0]
        else:              w = pos2[0]-pos1[0]
        if pos1[1]>pos2[1]:h = pos1[1]-pos2[1]
        else:              h = pos2[1]-pos1[1]
        dl = math.sqrt(w*w+h*h)
        return dl

    def ANGLE_TO(point1,point2):
        radians = math.atan2(point1[0]-point2[0],point1[1]-point2[1])
        angle = (360*radians)/(2*math.pi)
        if angle<0:
            angle=360+angle
        return angle

    def SRED(listing):
        sum = 0
        for num in listing: sum+=num
        return sum/len(listing)
        

class Vec2_:
    def __init__(self,points) -> None:
        self.vector = pygame.math.Vector2(points)
        
    def Rotate(self,angle):
        self.vector.rotate_ip(angle)

    def x(self):
        return self.vector.x

    def xx(self):
        return self.vector.xx

    def y(self):
        return self.vector.y

    def yy(self):
        return self.vector.yy

    def xy(self):
        return self.vector.xy

    def Normalize(self,New:bool=False):
        if New:
            return Vec2_( self.vector.normalize() )
        elif not New:
            self.vector = self.vector.normalize()

    def Rotate_rad(self,angle):
        self.vector.rotate_ip_rad(angle)

    def Angle_to(self,vector) -> float:
        ang = self.vector.angle_to(vector.vector)
        if ang-45<0: ang = 360+ang
        return ang - 45

    def Lenght(self) -> float:
        return self.vector.length()

    def Lenght_Squared(self) -> float:
        return self.vector.length_squared()

    def Is_normalised(self) -> bool:
        return self.vector.is_normalized()

    def Cross(self,vector):
        return self.vector.cross(vector.vector)

    def Scale(self,value):
        self.vector.scale_to_length(value)

    def Sum(self,vector):
        pos = [self.vector.x+vector.vector.x,self.vector.y+vector.vector.y]
        self.vector = pygame.math.Vector2(pos)









import GL,GL_VFX



 
win = GL.Display_init_(flags=GL.D_Scale)
 
p1 = GL_VFX.Particles.Rect(
    surf=win,
    draw_surf=win,
    rect=[-1,-1,1,1],
    shape_data=['c',20,[[200,10,240]]],
    circle_speed=2
)

while win.CEUF():
    p1.Set_position(GL.Open_mouse.GET_POSITION())
    p1.P()

    
    


import colorama


print(colorama.Fore.BLUE+'Pygl version - (21.7)'+colorama.Fore.RESET)
print(colorama.Fore.CYAN+colorama.Style.BRIGHT +
      'My Github --- ( https://github.com/dedisvoin/GRPgraph-versions )'+colorama.Fore.RESET, colorama.Style.RESET_ALL)

__version__ = '21.7'



from typing import Union
from pygl_nf.GL_MATH import Vec2_, Math_
from pygl_nf import GL_MATH
import random 
from copy import copy
from pygl_nf.GL import Text_
from pygl_nf.GL import Color_
from pygl_nf import GL
import pygame

class Particles():
    class Point():
        def __init__(self,surf,point_pos, gravity, shape_data, life_time = 2, speed=1, particle_count=5, spawn_time = 10, size_resize=True, size_deller = 0.1, max_particles=300) -> None:
            self.point_pos = Vec2_(pos=point_pos)
            self.gravity = Vec2_(pos=gravity)
            self.spawn_time = spawn_time
            self.timer = 0
            self.speed = speed
            self.life_time = life_time
            self.surf = surf
            self.max_particles = max_particles
            
            self.size_resize = size_resize
            self.size_deller = size_deller

            self.shape = shape_data[0]
            if self.shape == 'i':
                self.img = shape_data[3]
                self.i = self.img
            self.color = shape_data[1]
            self.size =  shape_data[2]


            self.particle_count = particle_count
            
            self.particles = []
        def Emiter(self):
            self.timer+=1
            if self.timer%self.spawn_time==0:

              for i in range(self.particle_count):
                    sped_vector = Vec2_(pos = [random.randint(-self.speed,self.speed)*random.random(),random.randint(-self.speed,self.speed)*random.random()] )
                    particle = [self.point_pos,sped_vector,self.life_time,self.size]
                    self.particles.append(particle)
        def Lifeter(self):
            for i in range(len(self.particles)):
                self.particles[i][2]-=0.1
                if self.particles[i][0].y>self.surf.GET_WIN_HEIGHT() or self.particles[i][0].y<0 or self.particles[i][0].x<0 or self.particles[i][0].x>self.surf.GET_WIN_WIDTH():
                    del self.particles[i]
                    break
                if self.particles[i][2]<=0:
                    del self.particles[i]
                    break   
        def Xclean(self):
            for i in range(len(self.particles)):
                if self.particles[i][3]<=0:
                    del self.particles[i]
                    break
                if random.randint(0,1)==1:
                    del self.particles[i]
                    break
            if len(self.particles)>self.max_particles:
                del self.particles[0:50]
        def PCount(self,pos):
            Text_(str(len(self.particles)),True,'black','arial',20,pos,SURF=self.surf.screen).RENDER()       
        def Focus(self):
            for i in range(len(self.particles)):
                self.particles[i][1] = self.particles[i][1].SUM(self.gravity)
                self.particles[i][0] = self.particles[i][0].SUM(self.particles[i][1])
                if self.size_resize:
                    self.particles[i][3]-=self.size_deller              
        def Render(self):
            for i in range(len(self.particles)):
                if self.shape == 'r':
                    if self.particles[i][2]>0:
                        self.surf.GL.Rect(self.color,self.particles[i][0].pos1,[self.particles[i][3],self.particles[i][3]],0,'s','D')
                
                elif self.shape == 'c':
                    if self.particles[i][2]>0:
                        self.surf.GL.Circle(self.color,self.particles[i][0].pos1,self.particles[i][3],0,'s','D')

                elif self.shape == 'i':
                    if self.particles[i][2]>0:
                        self.i = self.img
                        self.i.Set_pos(self.particles[i][0].pos1)
                        self.i.Scale([self.size,self.size])
                        self.i.Draw(self.surf.screen)
        def Set_position(self,pos):
            self.point_pos = Vec2_(pos=pos)
        def Set_color(self,color):
            self.color = color
        def Set_gravity(self,gravity):
            self.gravity = Vec2_(pos=gravity)
        def Set_speed(self,speed):
            self.speed = speed
        def Set_particle_count(self,count):
            self.particle_count = count
        def Set_spawn_time(self,time):
            self.spawn_time = time
        def Set_size_deller(self,deller):
            self.size_deller = deller
        def Set_size(self,size):
            self.size = size

    class Rect():
        def __init__(self,
                        surf:GL.Display_init_,
                        rect,
                        shape_data,
                        color_randoming=False,
                        color_index=0,
                        circle_speed = 1,
                        vector_speed = [0,0],
                        size_deller = 1,
                        return_size=False,
                        size_resize = True,
                        particle_count = 5,
                        max_particle = 300,
                        dell_count = 100,
                        gravity=[0,0],
                        life_time = 200,
                        life_dell_count = 5,
                        life_delta = 0.01,
                        spawn_time = 1,
                        spawn_delta = 1,
                        image_rotating = False,
                        image_rotating_delta = 1,
                        randomig_iamge = False,
                        lighting = False,
                        lighting_color = (20,20,20),
                        light_radius_resize = False,
                        light_radius = 10,
                        light_radius_clip_in_shape_rad = False,
                        light_alpha = 255,
                        light_color_dispers = 200,
                        coed_part_lines = False,
                        coed_part_lines_centered = False,
                        coed_part_lines_add = 0,
                        coed_part_lines_save_end_pos = False,
                        rotate_in_center = False,
                        rotating_in_center_angle = 0,
                        rotate_gravity = False,
                        rotating_gravity_angle = 0,
                        image_rotate_in_position = False,
                        image_rotate_in_vector = False,
                        image_rotate_in_position_angle = 0,
                        teni = False,
                        teni_vector = [1,1],
                        teni_color = (0,0,0),
                        teni_fill = True,
                        teni_size_obvodka = 5,
                        size_randoming = False,
                        size_min_max = [],
                        size_resize_timer = 1,
                        vector_speed_randoming_angle = 90,
                        shader = 'pos',
                        draw_surf=None
                        ) -> None:
                self.shader = shader
                self.vector_speed_randoming_angle = vector_speed_randoming_angle
                self.size_resize_timer = size_resize_timer
                self.size_randoming = size_randoming
                self.size_min_max = size_min_max
                self.teni_size_obvodka = teni_size_obvodka
                self.teni_fill = teni_fill
                self.teni_color = teni_color
                self.vector_teni = teni_vector
                self.teni = teni
                self.start_render = True
                self.start_emit = True
                self.start_focus = True
                self.start_delete = True
                self.rotating_gravity_angle = rotating_gravity_angle
                self.rotate_gravity = rotate_gravity
                self.rotating_in_center_angle = rotating_in_center_angle
                self.rotate_in_center = rotate_in_center
                self.light_color_dispers = light_color_dispers
                self.coed_part_lines_save_end_pos = coed_part_lines_save_end_pos ; self.end_line_pos = []
                self.coed_part_lines_add = coed_part_lines_add
                self.coed_part_lines_centered  = coed_part_lines_centered
                self.soed_part_lines = coed_part_lines
                self.light_alpha = light_alpha
                self.light_radius_clip_in_shape_rad = light_radius_clip_in_shape_rad
                self.light_radius_resize = light_radius_resize
                self.light_radius = light_radius
                self.lighting = lighting
                self.lighting_color = lighting_color
                self.image_rotate_in_position = image_rotate_in_position
                self.image_rotate_in_vector = image_rotate_in_vector
                self.image_rotate_in_position_angle = image_rotate_in_position_angle
                self.image_rotating = image_rotating
                self.image_rotating_delta = image_rotating_delta
                self.rect = rect
                self.posx = self.rect[0]
                self.posy = self.rect[1]
                self.width = self.rect[2]
                self.height = self.rect[3]
                self.circle_speed = circle_speed
                self.size_deller = size_deller
                self.partcle_count = particle_count
                self.max_particle = max_particle
                self.size_resize = size_resize
                self.return_size = return_size
                self.gravity = Vec2_(gravity)
                self.dell_count = dell_count
                self.vector_speed = vector_speed
                self.life_time = life_time
                self.life_dell_count = life_dell_count
                self.life_delta = life_delta
                self.spawn_time = spawn_time
                self.spawn_delta = spawn_delta
                self.time = 0
                self.surf = surf
                self.move_point = []
                self.move_point_set = False
                self.move_point_speed = 100
                self.move_radius = 100
                self.move_radius_delta_x = 1
                self.move_radius_delta_y = 1
                self.color_randoming = color_randoming
                self.color_index = color_index
                self.randoming_image = randomig_iamge
                self.end_mouse_pos = 0
                self.draw_surf = draw_surf


                self.phisics_color = None


                self.shape_data = shape_data
                self.shape = self.shape_data[0]
                self.shape_color = self.shape_data[2]
                if self.shape == 'i':
                    self.img = self.shape_data[2]
                    self.orig_image = self.img
                    

                self.shape_max_size = self.shape_data[1]


                if self.return_size:self.shape_widt = 0
                else:self.shape_widt = self.shape_data[1]
                


                self.particles = []




                self.id = random.randint(1,1000000000)
        def Phisics(self,color,randoming_vector,vector=[10,10],tyga=[0,1],density_y=0.9,density_x=0.6):
            self.phisics_color = color
            tyga = GL_MATH.Vec2_(tyga)
            
            for i in range(len(self.particles)):
                self.particles[i][1].vector+=tyga.vector
                if self.shape == 'c':
                    if (self.surf.IN_WINDOW([int(self.particles[i][0].vector.x+self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y)])  and    
                        self.surf.IN_WINDOW([int(self.particles[i][0].vector.x-self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y)])  and
                        self.surf.IN_WINDOW([int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][2]+self.particles[i][1].vector.y)])  and
                        self.surf.IN_WINDOW([int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y-self.particles[i][2]+self.particles[i][1].vector.y)])     ):

                        if randoming_vector==False:
                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][2]+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y-self.particles[i][2]+self.particles[i][1].vector.y) ) == color:
                                self.particles[i][1].vector.y = -(self.particles[i][1].vector.y*density_y)
                                self.particles[i][1].vector.x = (self.particles[i][1].vector.x*density_x)

                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x-self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color:
                                self.particles[i][1].vector.x = -(self.particles[i][1].vector.x*density_x)
                            
                        
                                
                        else:
                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][2]+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y-self.particles[i][2]+self.particles[i][1].vector.y) ) == color:
                                self.particles[i][1].vector.y = -(self.particles[i][1].vector.y*density_y)
                                self.particles[i][1].vector.x += random.randint(-vector[0],vector[0])

                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x-self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color:
                                self.particles[i][1].vector.x = -(self.particles[i][1].vector.x*density_x)
                                self.particles[i][1].vector.y += random.randint(-vector[1],vector[1])
            

        
        def mult(self,num):
            if num<=0:
                num = 0
            return num

        def Set_move_point(self,pos,speed=100,move_radius=100,move_radius_delta_x=4,move_radius_delta_y=4):
                self.move_point = pos
                self.move_point_set = True
                self.move_point_speed = speed
                self.move_radius = move_radius
                self.move_radius_delta_x = move_radius_delta_x
                self.move_radius_delta_y = move_radius_delta_y

        def Emiter(self):
            if self.start_emit:
                self.time+=self.spawn_delta
                if self.time%self.spawn_time==0:
                    self.end_line_pos = [self.posx,self.posy]
                    self.posx = int(self.posx) ; self.posy = int(self.posy)
                    

                    for i in range(self.partcle_count):
                                pos = Vec2_([
                                    random.randint(self.posx,self.posx+self.width),
                                    random.randint(self.posy,self.posy+self.height)
                                ])
                                
                                self.cirkul_spee = Vec2_([
                                    random.randint(-10000,10000)/10000*self.circle_speed,
                                    random.randint(-10000,10000)/10000*self.circle_speed
                                ])
                                
                                self.vector_spee = Vec2_(self.vector_speed)
                                angle = random.randint(-self.vector_speed_randoming_angle,self.vector_speed_randoming_angle)
                                self.vector_spee.Rotate(angle)
                                
                                if self.shape != 'i':
                                    if self.color_randoming == True and type(self.shape_color)==list and len(self.shape_color)>1:
                                        color = self.shape_color[random.randint(0,len(self.shape_color)-1)]
                                    else:
                                        color = self.shape_color[self.color_index]
                                    if self.shape_color == 'r':
                                        color =Color_._rgb('r','r','r').COLOR
                                    if self.size_randoming:
                                        self.shape_widt = random.randint(self.size_min_max[0],self.size_min_max[1])
                                    if self.shader == 'rg':
                                        color = [pos.vector.x/self.surf.GET_WIN_WIDTH()*255,pos.vector.y/self.surf.GET_WIN_HEIGHT()*255,0]
                                    elif self.shader == 'rb':
                                        color = [pos.vector.x/self.surf.GET_WIN_WIDTH()*255,0,pos.vector.y/self.surf.GET_WIN_HEIGHT()*255]
                                    elif self.shader == 'bg':
                                        color = [0,pos.vector.x/self.surf.GET_WIN_WIDTH()*255,pos.vector.y/self.surf.GET_WIN_HEIGHT()*255]

                                    particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,0,self.size_resize_timer]
                                                
                                else:
                                    color = None
                                    if self.image_rotating:
                                        rotate = random.randint(-self.image_rotating_delta,self.image_rotating_delta)
                                    else:
                                        rotate = 0
                                    
                                    if self.randoming_image:
                                        ri = random.randint(0,len(self.img)-1)
                                        img = self.img[ri]
                                        
                                        rect = self.img[ri].img.get_rect(center = pos.vector.xy)
                                        if self.size_randoming:
                                            self.shape_widt = random.randint(self.size_min_max[0],self.size_min_max[1])
                                        particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,rotate,0,rect,img]
                                    else:
                                        rect = self.img.img.get_rect(center = pos.vector.xy)
                                        if self.size_randoming:
                                            self.shape_widt = random.randint(self.size_min_max[0],self.size_min_max[1])
                                        particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,rotate,0,rect]

                                

                                self.particles.append(particle)

        def Render(self):
            if self.start_render:
                if self.teni:
                    for i in range(len(self.particles)):
                        if self.shape == 'c':
                            if self.teni_fill:
                                self.draw_surf.GL.Circle(self.teni_color,[self.particles[i][0].vector.x+self.vector_teni[0],self.particles[i][0].vector.y+self.vector_teni[1]],self.particles[i][2],0,self.surf.screen,'D')
                            else:
                                self.draw_surf.GL.Circle(self.teni_color,[self.particles[i][0].vector.x+self.vector_teni[0],self.particles[i][0].vector.y+self.vector_teni[1]],self.particles[i][2],self.teni_size_obvodka,self.surf.screen,'D')

                        elif self.shape == 'r':
                            if self.teni_fill:
                                self.draw_surf.GL.Rect(self.teni_color,[self.particles[i][0].vector.x-self.particles[i][2]/2+self.vector_teni[0],self.particles[i][0].vector.y-self.particles[i][2]/2+self.vector_teni[1]],[self.particles[i][2],self.particles[i][2]],0,self.surf.screen,'D')


                for i in range(len(self.particles)):

                    if self.shape == 'r':

                        self.draw_surf.GL.Rect(self.particles[i][5],[self.particles[i][0].vector.x-self.particles[i][2]/2,self.particles[i][0].vector.y-self.particles[i][2]/2],[self.particles[i][2],self.particles[i][2]],0,self.surf.screen,'D')
                    elif self.shape == 'c':

                        self.draw_surf.GL.Circle(self.particles[i][5],self.particles[i][0].vector.xy,self.particles[i][2],0,self.surf.screen,'D')
                    elif self.shape == 'i':
                        if self.randoming_image:
                            
                            image = copy(self.particles[i][9])
                        else:
                            image = copy(self.img)
                        
                        
                        image.Set_pos([self.particles[i][0].vector.x-self.particles[i][2]/2,self.particles[i][0].vector.y-self.particles[i][2]/2])
                        if self.particles[i][2]>0:
                            image.Scale([self.particles[i][2],self.particles[i][2]])
                            new_image = pygame.transform.rotate(image.img,self.particles[i][7])
                            self.particles[i][8] = new_image.get_rect(center=[self.particles[i][8].center[0]+self.particles[i][0].vector.x-self.particles[i][8].center[0],self.particles[i][8].center[1]+self.particles[i][0].vector.y-self.particles[i][8].center[1]])
                            
                            
                            
                            self.surf.screen.blit(new_image,self.particles[i][8])


                    if self.lighting:
                        
                        if self.light_radius_resize:
                            s1 = GL.Surfases_([self.particles[i][2]*4,self.particles[i][2]*4],[self.particles[i][0].vector.x-self.particles[i][2]*2,self.particles[i][0].vector.y-self.particles[i][2]*2],None,self.light_alpha,GL.S_Blend)
                            if self.lighting_color=='shape_color':self.draw_surf.GL.Circle([self.mult(self.particles[i][5][0]-self.light_color_dispers),self.mult(self.particles[i][5][1]-self.light_color_dispers),self.mult(self.particles[i][5][2]-self.light_color_dispers)],[self.particles[i][2]*2,self.particles[i][2]*2],self.particles[i][2]*2,0,s1.screen,'D')
                            else:self.draw_surf.GL.Circle(self.lighting_color,[self.particles[i][2]*2,self.particles[i][2]*2],self.particles[i][2]*2,0,s1.screen,'D')
                            s1.DRAW_(self.surf.screen)
                        else:
                            if not self.light_radius_clip_in_shape_rad:
                                s1 = GL.Surfases_([self.light_radius*2,self.light_radius*2],[self.particles[i][0].vector.x-self.light_radius,self.particles[i][0].vector.y-self.light_radius],None,self.light_alpha,GL.S_Blend)
                                if self.lighting_color=='shape_color':self.draw_surf.GL.Circle([self.mult(self.particles[i][5][0]-self.light_color_dispers),self.mult(self.particles[i][5][1]-self.light_color_dispers),self.mult(self.particles[i][5][2]-self.light_color_dispers)],[self.light_radius,self.light_radius],self.light_radius,0,s1.screen,'D')  
                                else:self.draw_surf.GL.Circle(self.lighting_color,[self.light_radius,self.light_radius],self.light_radius,0,s1.screen,'D')  
                                s1.DRAW_(self.surf.screen)
                            else:
                                s1 = GL.Surfases_([self.mult(self.particles[i][2]*2+self.light_radius*2),self.mult(self.particles[i][2]*2+self.light_radius*2)],[self.particles[i][0].vector.x-(self.light_radius+self.particles[i][2]),self.particles[i][0].vector.y-(self.light_radius+self.particles[i][2])],None,self.light_alpha,GL.S_Blend)
                                if self.lighting_color=='shape_color':self.draw_surf.GL.Circle([self.mult(self.particles[i][5][0]-self.light_color_dispers),self.mult(self.particles[i][5][1]-self.light_color_dispers),self.mult(self.particles[i][5][2]-self.light_color_dispers)],[self.light_radius+self.particles[i][2],self.light_radius+self.particles[i][2]],self.light_radius+self.particles[i][2],0,s1.screen,'D')  
                                else:self.draw_surf.GL.Circle(self.lighting_color,[self.light_radius+self.particles[i][2],self.light_radius+self.particles[i][2]],self.light_radius+self.particles[i][2],0,s1.screen,'D')  
                                s1.DRAW_(self.surf.screen)
        
                    if self.soed_part_lines:
                        if self.coed_part_lines_centered:
                            
                                self.draw_surf.GL.Line(self.particles[i][5],self.particles[i][0].vector.xy,self.end_line_pos,int(self.particles[i][2]+self.coed_part_lines_add),self.surf.screen,'S','D')
                        else:
                            if len(self.particles)-i<len(self.particles):
                                self.draw_surf.GL.Line(self.particles[i][5],self.particles[i][0].vector.xy,self.particles[i-1][0].vector.xy,int(self.particles[i][2]+self.coed_part_lines_add),self.surf.screen,'S','D')

        def Focus(self):
            if self.start_focus:
                if self.rotate_gravity:
                    self.gravity.Rotate(self.rotating_gravity_angle)
                for i in range(len(self.particles)):

                    if self.size_resize_timer!=None:
                        self.particles[i][7]-=1


                    if self.rotate_in_center:
                        self.particles[i][1].Rotate(self.rotating_in_center_angle)
                    
                    self.particles[i][1].vector += self.gravity.vector

                    if self.move_point_set:    
                        sx = -(self.particles[i][0].vector.x-self.move_point[0])/self.move_point_speed
                        sy = -(self.particles[i][0].vector.y-self.move_point[1])/self.move_point_speed
                        if GL_MATH.Math_().RAST(self.particles[i][0].vector.xy,self.move_point)<self.move_radius:
                            sx*=self.move_radius_delta_x
                            sy*=self.move_radius_delta_y
                        
                        sv = GL_MATH.Vec2_([sx,sy])
                        self.particles[i][0].vector += sv.vector
                    if self.image_rotating:
                        self.particles[i][7]+=self.particles[i][6]
                    
                        

                    self.particles[i][1].vector+=self.particles[i][3].vector
                    self.particles[i][0].vector += self.particles[i][1].vector

                    self.particles[i][4]-=self.life_delta
                    if self.particles[i][7]<=0:
                        if self.size_resize:
                            if self.return_size:  
                                if self.particles[i][2]<=self.shape_max_size+10:
                                    self.particles[i][2]+=self.size_deller
                            else:
                                self.particles[i][2]-=self.size_deller

                    if self.image_rotate_in_position:
                        center_pos = GL_MATH.Vec2_([self.posx-self.particles[i][0].vector.x,self.posy-self.particles[i][0].vector.y])
                        self.particles[i][7]=-int(self.particles[i][0].Angle_to(center_pos))+300

                    if self.image_rotate_in_vector:
                        
                        #speed_pos = GL_MATH.Vec2_([self.particles[i][1].vector.x+self.particles[i][0].vector.x,self.particles[i][1].vector.y+self.particles[i][0].vector.y])
                        self.particles[i][7]=-int(self.particles[i][0].Angle_to(self.particles[i][1]))+125
        
        def Xclean(self):
            if self.start_delete:
                for i in range(len(self.particles)):
                    if self.particles[i][1].vector.xy==[0,0]:
                        del self.particles[i]
                        break

                for i in range(len(self.particles)):
                    if self.surf.IN_WINDOW(self.particles[i][0].vector.xy):
                        pass
                    else:
                        del self.particles[i]
                        break

                for i in range(len(self.particles)):
                    if self.particles[i][2]<=0:
                        del self.particles[i]
                        break

                for i in range(len(self.particles)):
                    if self.particles[i][4]<0:
                        del self.particles[-self.life_dell_count:]
                        break

                if self.return_size:
                    for i in range(len(self.particles)):
                        if self.particles[i][2]>self.shape_max_size:
                            del self.particles[i]
                            break

                

                if len(self.particles)>self.max_particle:
                    del self.particles[1:self.dell_count]
                              
        def PCount(self,pos):
                Text_(str(len(self.particles)),True,'black','arial',15,pos,SURF=self.surf.screen).RENDER()

        def Set_width(self,width):
                self.width = width

        def Set_height(self,height):
                self.height = height

        def Set_position(self,position):
                self.posx = position[0]
                self.posy = position[1]

        def Move(self,speed):
            self.posx+=speed[0]
            self.posy+=speed[1]
            
            for i in range(len(self.particles)):
                self.particles[i][0].vector.x+=speed[0]
                self.particles[i][0].vector.y+=speed[1]
        
        def P(self):
            self.Emiter()
            self.Focus()
            self.Render()
            self.Xclean()

        def Get_data(self):
            print(f'''particle emmiter
| id {self.id},
| particle_count {len(self.particles)},
| rect [ x:{self.posx},y:{self.posy},w:{self.width},h:{self.height} ],
| shape [ type:'{self.shape}',size:{self.shape_widt} ],
| shape colors {self.shape_color},
| color randoming {self.color_randoming}
| color index {self.color_index}
| gravity {self.gravity.vector.xy},
| vector speed {self.vector_speed},
| circle speed {self.circle_speed},
| rotating in center [bool:{self.rotate_in_center},angle:{self.rotating_in_center_angle}],
| rotating gravity   [bool:{self.rotate_gravity},angle:{self.rotating_gravity_angle}],
| lightning {self.lighting},
    | light dispers {self.light_color_dispers},
    | light radius {self.light_radius},
    | light color [ {self.lighting_color} ],
    | light radius clip in shape radius {self.light_radius_clip_in_shape_rad},
    | light radius resize {self.light_radius_resize}


''')

    class Bax():
        def __init__(self,surf,position,max_size,min_size,start_size,size_delta,size_delta2,color,type='c',start_rad=10,rad_delta=0,zikl=False,teni=False):
            self.pos = position
            self.max_size = max_size
            self.min_size = min_size
            self.start_size = start_size
            self.ss = self.start_size
            self.size_delta = size_delta
            self.size_delta2 = size_delta2
            self.sd2 = self.size_delta2
            self.teni = teni

            self.start_rad = start_rad
            self.sr = self.start_rad
            self.rad_delta = rad_delta

            self.zikl = zikl


            self.color = color
            self.type = type
            self.surf = surf
        def Move(self,speed):
            self.pos[0]+=speed[0]
            self.pos[1]+=speed[1]

        def Render(self):
            if self.type == 'c':
                if self.start_rad>0:
                    if self.teni:
                        self.surf.GL.Circle('black',[self.pos[0]+4,self.pos[1]+4],self.start_size,self.start_rad,'s','D')
                    self.surf.GL.Circle(self.color,self.pos,self.start_size,self.start_rad,'s','D')
            elif self.type == 'r':
                if self.start_rad>0:
                    self.surf.GL.Rect(self.color,[self.pos[0]-self.start_size/2,self.pos[1]-self.start_size/2],[self.start_size,self.start_size],self.start_rad,'s','D')

        def Focus(self):
            self.start_rad+=self.rad_delta
            
            self.size_delta2+=self.sd2
            if self.start_size<=self.max_size and self.start_size>self.min_size:
                self.start_size+=self.size_delta+self.size_delta2
            

            if self.zikl:
                if self.start_size<self.min_size or self.start_size>self.max_size:
                    self.start_size = self.ss
                    self.size_delta2 = self.sd2
                    self.start_rad = self.sr


from setuptools import setup
requirements = ["Pygame==2.1.1", "keyboard",
                "pygame_widgets", "colorama", "numpy", "colored","mouse"]
setup(name='pygl_nf',
      version='21.7',
      description='small and compact graphick distributions',
      packages=['pygl_nf'],
      author_email='pvana621@gmail.com',
      install_requires=requirements,
      zip_safe=False)



from pygl_nf import GL
from colorama import Fore as FO
import sys
import time
import os

initing = False
def Init():
    global initing

    for i in range(5): 
        os.system('cls')
        print(FO.CYAN+'Prefabs Initing'+FO.RESET,end='')
        print(FO.CYAN+'.'*(i+1)+FO.RESET)
        time.sleep(0.1)
    os.system('cls')

    if os.path.exists('Prefabs'):
        initing = True
        print(FO.GREEN+'''[ Success ]'''+FO.RESET)
        print(FO.GREEN+'Prefabs Init!'+FO.RESET)
    else:
        print(FO.RED+'''[ Error ]'''+FO.RESET)
        print(FO.RED+'Prefabs Not Init!'+FO.RESET)
        print(FO.YELLOW+'Prefab file not found'+FO.RESET)
        print(FO.CYAN+'Download -- https://github.com/dedisvoin/GRPgraph-versions'+FO.RESET)
    
    
    


class Sound(object):
    def __init__(self):
        global initing
        if initing==False:
            print(FO.RED+'''[ Error ]'''+FO.RESET)
            print(FO.RED+'Prefabs Not Init!'+FO.RESET)
            sys.exit()
            
    class Jamps(object):
        def __init__(self):
            self.jamp1 = 'Prefabs\Sounds\Jamps\Jump1.wav'
            self.jamp2 = 'Prefabs\Sounds\Jamps\Jump2.wav'
            self.jamp3 = 'Prefabs\Sounds\Jamps\Jump3.wav'
            self.jamp4 = 'Prefabs\Sounds\Jamps\Jump4.wav'
            self.jamp5 = 'Prefabs\Sounds\Jamps\Jump5.wav'
            self.jamp6 = 'Prefabs\Sounds\Jamps\Jump6.wav'
            self.jamp7 = 'Prefabs\Sounds\Jamps\Jump7.wav'
            self.jamp8 = 'Prefabs\Sounds\Jamps\Jump8.wav'
            self.jamp9 = 'Prefabs\Sounds\Jamps\Jump9.wav'
            self.jamp10 = 'Prefabs\Sounds\Jamps\Jump10.wav'


            self._Jamp_Sounds = [
                self.jamp1,self.jamp2,self.jamp3,self.jamp4,self.jamp5,
                self.jamp6,self.jamp7,self.jamp8,self.jamp9,self.jamp10
            ]

        def GET_SOUND_FOR_NAME(self,Sound_file):
            sound = GL.Sound_mixer_(Sound_file)
            return sound

        def GET_SOUND_FOR_INDEX(self,Index):
            if Index > len(self._Jamp_Sounds):
                print (FO.RED+"Sound index out of range"+FO.RESET)
                sys.exit()
            sound = GL.Sound_mixer_(self._Jamp_Sounds[Index-1])
            return sound

    class Pickaps_and_Coin(object):
        def __init__(self):
            self.pickup1 = 'Prefabs\Sounds\Pickuping\Pickup_Coin1.wav'
            self.pickup2 = 'Prefabs\Sounds\Pickuping\Pickup_Coin2.wav'
            self.pickup3 = 'Prefabs\Sounds\Pickuping\Pickup_Coin3.wav'
            self.pickup4 = 'Prefabs\Sounds\Pickuping\Pickup_Coin4.wav'
            self.pickup5 = 'Prefabs\Sounds\Pickuping\Pickup_Coin5.wav'
            self.pickup6 = 'Prefabs\Sounds\Pickuping\Pickup_Coin6.wav'
            self.pickup7 = 'Prefabs\Sounds\Pickuping\Pickup_Coin7.wav'
            self.pickup8 = 'Prefabs\Sounds\Pickuping\Pickup_Coin8.wav'
            self.pickup9 = 'Prefabs\Sounds\Pickuping\Pickup_Coin9.wav'
            self.pickup10 = 'Prefabs\Sounds\Pickuping\Pickup_Coin10.wav'


            self._Pickup_Sounds = [
                self.pickup1,self.pickup2,self.pickup3,self.pickup4,self.pickup5,
                self.pickup6,self.pickup7,self.pickup8,self.pickup9,self.pickup10
            ]

        def GET_SOUND_FOR_NAME(self,Sound_file):
            sound = GL.Sound_mixer_(Sound_file)
            return sound

        def GET_SOUND_FOR_INDEX(self,Index):
            if Index > len(self._Pickup_Sounds):
                print (FO.RED+"Sound Index out of range"+FO.RESET)
                sys.exit()
            sound = GL.Sound_mixer_(self._Pickup_Sounds[Index-1])
            return sound


    class Laser_and_Shoot(object):
        def __init__(self):
            self.Laser1 = 'Prefabs\Sounds\Laser\Laser_Shoot1.wav'
            self.Laser2 = 'Prefabs\Sounds\Laser\Laser_Shoot2.wav'
            self.Laser3 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser4 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser5 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser6 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser7 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser8 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser9 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'
            self.Laser10 = 'Prefabs\Sounds\Laser\Laser_Shoot3.wav'

            self._Laser_Sounds = [
                self.Laser1,self.Laser2,self.Laser3,self.Laser4,self.Laser5,
                self.Laser6,self.Laser7,self.Laser8,self.Laser9,self.Laser10
            ]

        def GET_SOUND_FOR_NAME(self,Sound_file):
            sound = GL.Sound_mixer_(Sound_file)
            return sound

        def GET_SOUND_FOR_INDEX(self,Index):
            if Index > len(self._Laser_Sounds):
                print(FO.RED+'Sound Index out of range'+FO.RESET)
                sys.exit()
            sound = GL.Sound_mixer_(self._Laser_Sounds[Index-1])
            return sound


    class Explosion(object):
        def __init__(self):
            self.explosion1 = 'Prefabs\Sounds\Explosion\Explosion1.wav'
            self.explosion2 = 'Prefabs\Sounds\Explosion\Explosion2.wav'
            self.explosion3 = 'Prefabs\Sounds\Explosion\Explosion3.wav'
            self.explosion4 = 'Prefabs\Sounds\Explosion\Explosion4.wav'
            self.explosion5 = 'Prefabs\Sounds\Explosion\Explosion5.wav'
            self.explosion6 = 'Prefabs\Sounds\Explosion\Explosion6.wav'
            self.explosion7 = 'Prefabs\Sounds\Explosion\Explosion7.wav'
            self.explosion8 = 'Prefabs\Sounds\Explosion\Explosion8.wav'
            self.explosion9 = 'Prefabs\Sounds\Explosion\Explosion9.wav'
            self.explosion10 = 'Prefabs\Sounds\Explosion\Explosion10.wav'
            self.explosion11 = 'Prefabs\Sounds\Explosion\Explosion11.wav'
            self.explosion12 = 'Prefabs\Sounds\Explosion\Explosion12.wav'
            self.explosion13 = 'Prefabs\Sounds\Explosion\Explosion13.wav'
            self.explosion14 = 'Prefabs\Sounds\Explosion\Explosion14.wav'
            self.explosion15 = 'Prefabs\Sounds\Explosion\Explosion15.wav'

            self._Explosion_Sounds = [
                self.explosion1,self.explosion2,self.explosion3,self.explosion4,self.explosion5,
                self.explosion6,self.explosion7,self.explosion8,self.explosion9,self.explosion10,
                self.explosion11,self.explosion12,self.explosion13,self.explosion14,self.explosion15
            ]
            
        def GET_SOUND_FOR_NAME(self,Sound_file):
            sound = GL.Sound_mixer_(Sound_file)
            return sound

        def GET_SOUND_FOR_INDEX(self,Index):
            if Index > len(self._Explosion_Sounds):
                print(FO.RED+'Sound Index out of range'+FO.RESET)
                sys.exit()
            sound = GL.Sound_mixer_(self._Explosion_Sounds[Index-1])
            return sound

        


        